<?xml version="1.0"?>
<!--
 *
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @version $Id: pdd.xsl 1574 2011-07-06 23:16:29Z guard $
 * @todo #18 We need to implement some sort of unit testing for
 *           the produced XHTML and JavaScript. Maybe JUnit shall
 *           be used. Without such testing we don't know whether the
 *           produced output is valid or not.
 *
 -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output method="html" indent="yes"
        doctype-public="-//W3C//DTD XHTML 1.1//EN"
        doctype-system="http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd" />

    <xsl:template match="/">
        <html>
            <xsl:call-template name="head" />
            <body>
                <xsl:apply-templates select="/pdd/wp" />
            </body>
            <xsl:call-template name="tail" />
        </html>
    </xsl:template>

    <xsl:template match="wp">
        <div class="wp">

            <!-- Marker of this WP -->
            <span>
                <xsl:attribute name="class">
                    <xsl:text>item</xsl:text>
                    <xsl:choose>
                        <xsl:when test="not(issue)">
                            <xsl:text> unassigned</xsl:text>
                        </xsl:when>
                    </xsl:choose>
                </xsl:attribute>
                <xsl:value-of select="@marker" />
            </span>

            <!-- Estimate of this WP and dependents -->
            <span class="marker">
                <xsl:text>&#x2666;</xsl:text>
                <xsl:choose>
                    <xsl:when test="@estimate &gt; 0">
                        <xsl:call-template name="hours">
                            <xsl:with-param name="mins" select="@estimate"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>?</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:choose>
                    <xsl:when test="count(dependants/wp)">
                        <xsl:text>+</xsl:text>
                        <xsl:choose>
                            <xsl:when test="sum(dependants//@estimate)">
                                <xsl:call-template name="hours">
                                    <xsl:with-param name="mins"
                                        select="sum(dependants//@estimate)"/>
                                </xsl:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:text>?</xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                </xsl:choose>
            </span>

            <!-- Pointer down to dependents -->
            <xsl:choose>
                <xsl:when test="count(dependants//wp)">
                    <span>
                        <xsl:attribute name="class">
                            <xsl:text>marker</xsl:text>
                            <xsl:choose>
                                <xsl:when test="count(dependants//wp[not(./issue)])">
                                    <xsl:text> emphasize</xsl:text>
                                </xsl:when>
                            </xsl:choose>
                        </xsl:attribute>
                        <xsl:text>&#x25BC;</xsl:text>
                        <xsl:value-of select="count(dependants//wp)" />
                    </span>
                </xsl:when>
            </xsl:choose>

            <!-- Pointer down to the issue assigned -->
            <xsl:choose>
                <xsl:when test="issue">
                    <span class="marker">
                        <xsl:text>&#x25B6;</xsl:text>
                        <a>
                            <xsl:attribute name="href">
                                <xsl:value-of select="issue/@href" />
                            </xsl:attribute>
                            <xsl:value-of select="issue" />
                        </a>
                        <xsl:text> </xsl:text>
                        <span class="person">
                            <xsl:value-of select="issue/@assignee" />
                        </span>
                    </span>
                </xsl:when>
            </xsl:choose>

            <!-- Subject of the WP -->
            <xsl:choose>
                <xsl:when test="subject">
                    <span class="element">
                        <xsl:value-of select="subject" />
                    </span>
                </xsl:when>
            </xsl:choose>

            <!-- Description of the WP -->
            <xsl:choose>
                <xsl:when test="description">
                    <div class="data"><xsl:value-of select="description" disable-output-escaping="yes" /></div>
                </xsl:when>
            </xsl:choose>

            <!-- Dependents of the WP -->
            <xsl:apply-templates select="dependants/wp" />

        </div>
    </xsl:template>

    <xsl:template name="hours">
        <xsl:param name="mins" as="xs:double"/>
        <xsl:choose>
            <xsl:when test="$mins &gt; 10">
                <xsl:variable name="hours" select="floor($mins div 60)" />
                <xsl:variable name="mod" select="($mins mod 60) div 60" />
                <xsl:choose>
                    <xsl:when test="$hours &gt; 0">
                        <xsl:value-of select="$hours" />
                    </xsl:when>
                </xsl:choose>
                <xsl:choose>
                    <xsl:when test="$mod &gt;= 0.875">
                        <xsl:text>&#x215E;</xsl:text> <!-- 7/8 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.75">
                        <xsl:text>&#190;</xsl:text> <!-- 3/4 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.666">
                        <xsl:text>&#x2154;</xsl:text> <!-- 2/3 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.625">
                        <xsl:text>&#x215D;</xsl:text> <!-- 5/8 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.5">
                        <xsl:text>&#189;</xsl:text> <!-- 1/2 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.375">
                        <xsl:text>&#x215C;</xsl:text> <!-- 3/8 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.333">
                        <xsl:text>&#x2153;</xsl:text> <!-- 1/3 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.25">
                        <xsl:text>&#188;</xsl:text> <!-- 1/4 -->
                    </xsl:when>
                    <xsl:when test="$mod &gt;= 0.125">
                        <xsl:text>&#x215B;</xsl:text> <!-- 1/8 -->
                    </xsl:when>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>&#x215F;</xsl:text> <!-- 1/ -->
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="head">
        <head>
            <title>PDD Report</title>
            <style>
                * {
                    font-family: 'Trebuchet MS';
                }
                body {
                    padding: 3em;
                }
                a, a:hover, a:visited {
                    color: #00a;
                }
                div.wp {
                    /* display: none; */
                    margin: 1.5em 0em 1.5em 2em;
                }
                div.visible {
                    display: block;
                }
                div.data {
                    margin: 0.5em 0em 1.5em 0em;
                    width: 40em;
                    white-space: pre-wrap;
                }
                span.item {
                    border: 1px solid gray;
                    background-color: lightgray;
                    padding: 0.2em 0.4em;
                    margin-right: 0.4em;
                    cursor: pointer;
                }
                span.unassigned {
                    border: 1px solid red;
                    background-color: #fcc;
                }
                span.author {
                    background-color: white;
                    color: gray;
                }
                span.element {
                    color: gray;
                    margin-left: 1em;
                }
                span.marker {
                    margin: 0em 0.4em;
                }
                span.emphasize {
                    color: red;
                }
                span.person {
                    color: #88f;
                }
                table#legend {
                    margin-top: 6em;
                }
                    table#legend td {
                        vertical-align: top;
                        padding: 0.5em 0em;
                    }
            </style>
            <script type="text/javascript"
                src="https://ajax.googleapis.com/ajax/libs/jquery/1.5.0/jquery.min.js"></script>
            <script type="text/javascript">
            //<![CDATA[
            $(document).ready(
                function() {
                    $("span.item").click(
                        function () {
                            $(this).siblings("div.wp").toggle();
                        }
                    );
                    $("body div.wp div.wp div.wp").hide();
                }
            );
            //]]>
            </script>
        </head>
    </xsl:template>

    <xsl:template name="tail">
        <table id="legend">
            <colgroup>
                <col style="width: 5em; text-align: right;" />
                <col style="width: 25em;" />
            </colgroup>
            <tbody>
                <tr>
                    <td><span class="item">...</span></td>
                    <td>is a unit of work in the implementation process, where
                        the hash code is the dynamic identifier of the unit
                        of work uniquely designating its location in the
                        SuD</td>
                </tr>
                <tr>
                    <td><span class="item unassigned">...</span></td>
                    <td>is a unit of work not assigned to anyone yet</td>
                </tr>
                <tr>
                    <td><span class="marker">&#x2666;2+3&#x2153;</span></td>
                    <td>estimate of the unit of work in hours, where the first
                        number is its own estimate and the second number
                        is the estimate of its subordinates</td>
                </tr>
                <tr>
                    <td><span class="marker">&#x25BC;3</span></td>
                    <td>identifies the amount of subordinate
                        units of work, highlighting the number in
                        red when some of them are not assigned to implementors yet</td>
                </tr>
                <tr>
                    <td><span class="marker">&#x25B6;123</span></td>
                    <td>denotes the ticket ID where this particular
                        unit of work is discussed and assigned to
                        the implementor</td>
                </tr>
            </tbody>
        </table>
        <p>
            The method and software used during generation of
            this report is called Puzzle Driven Development and
            is patent pending in United States Patent and Trademark
            Office (USPTO), patent application no.12/840,306 filled
            in July 2010. To avoid unintended
            <a href="http://en.wikipedia.org/wiki/Patent_infringement">patent infringement</a>
            and consecutive penalties kindly ask a written permission
            of the author beforehand: Yegor Bugayenko (<a href="mailto:yegor@rempl.com">yegor@rempl.com</a>).
        </p>
        <p>
            The page was generated on
            <xsl:value-of select="current-dateTime()"/>
        </p>
    </xsl:template>

</xsl:stylesheet>
